/*!
 * \file afilterctl_cfg.h
 * \brief Tool for controlling advanced filter - configuration of HW filter
 * \author Lukas Kekely <xkekel00@stud.fit.vutbr.cz>
 * \date 2012
 *
 * Copyright (C) 2012 Brno University of Technology
 *
 * LICENSE TERMS
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of the Company nor the names of its contributors
 *    may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * ALTERNATIVELY, provided that this notice is retained in full, this
 * product may be distributed under the terms of the GNU General Public
 * License (GPL), in which case the provisions of the GPL apply INSTEAD
 * OF those given above.
 *
 * This software is provided ``as is'', and any express or implied
 * warranties, including, but not limited to, the implied warranties of
 * merchantability and fitness for a particular purpose are disclaimed.
 * In no event shall the company or contributors be liable for any
 * direct, indirect, incidental, special, exemplary, or consequential
 * damages (including, but not limited to, procurement of substitute
 * goods or services; loss of use, data, or profits; or business
 * interruption) however caused and on any theory of liability, whether
 * in contract, strict liability, or tort (including negligence or
 * otherwise) arising in any way out of the use of this software, even
 * if advised of the possibility of such damage.
 *
 * $Id: afilterctl_cfg.h 2428 2012-09-21 05:56:02Z xkekel00 $
 *
 */

#ifndef AFILTERCTL_CFG_H_INCLUDED
#define AFILTERCTL_CFG_H_INCLUDED

/// MUST BE CONSISTENT WITH filter_core_config.vhd !!!
#define RULES_DATA_WIDTH             16
#define FILTER_IP_WIDTH             128
#define AB_RULES_ITEMS              512
#define AB_RULES_TABLES               3
#define CD_RULES_ITEMS              512
#define CD_RULES_TABLES               3
#define EF_RULES_ITEMS              512
#define EF_RULES_TABLES               3
#define G_RULES_ITEMS                 0
#define G_RULES_TABLES                0
#define HI_RULES_ITEMS              512
#define HI_RULES_TABLES               2


/// AUTOMATICLY COMPUTED PART !!!
#define __my_max(a,b) ((a)>(b))?(a):(b)
#define __my_log2(x)    (((x)<=0x1)?0:\
                        (((x)<=0x2)?1:\
                        (((x)<=0x4)?2:\
                        (((x)<=0x8)?3:\
                        (((x)<=0x10)?4:\
                        (((x)<=0x20)?5:\
                        (((x)<=0x40)?6:\
                        (((x)<=0x80)?7:\
                        (((x)<=0x100)?8:\
                        (((x)<=0x200)?9:\
                        (((x)<=0x400)?10:\
                        (((x)<=0x800)?11:\
                        (((x)<=0x1000)?12:\
                        (((x)<=0x2000)?13:\
                        (((x)<=0x4000)?14:\
                        (((x)<=0x8000)?15:\
                        (((x)<=0x10000)?16:\
                        (-1))))))))))))))))))
static const int FILTER_IP_WORDS=((FILTER_IP_WIDTH-1)/32)+1;
static const int FILTER_IV_WORDS=__my_max(__my_max(__my_max(AB_RULES_TABLES,EF_RULES_TABLES),__my_max(HI_RULES_TABLES,G_RULES_TABLES)),CD_RULES_TABLES);
static const int FILTER_RULES_TABLES[]={AB_RULES_TABLES, CD_RULES_TABLES, EF_RULES_TABLES, G_RULES_TABLES, HI_RULES_TABLES};
static const int FILTER_RULES_ITEMS[]={AB_RULES_ITEMS, CD_RULES_ITEMS, EF_RULES_ITEMS, G_RULES_ITEMS, HI_RULES_ITEMS};
static const int FILTER_RULES_ITEMS_WIDTH[]={__my_log2(AB_RULES_ITEMS), __my_log2(CD_RULES_ITEMS), __my_log2(EF_RULES_ITEMS), __my_log2(G_RULES_ITEMS), __my_log2(HI_RULES_ITEMS)};
static const uint32_t FILTER_NOMATCH=(RULES_DATA_WIDTH<32)?(((uint32_t)1)<<RULES_DATA_WIDTH)-1:0xFFFFFFFF;

#endif
