/**
 * ccci_hdr.h
 * Copyright (C) 2012, 2013 Jan Viktorin
 */

#ifndef CCCI_HDR_H
#define CCCI_HDR_H

#include "compat_endian.h"
#include <inttypes.h>
#include <string.h>

/**
 * CCCI header representation.
 */
union ccci_hdr {
	uint8_t  raw[6];
	uint16_t raw16[3];
};

/**
 * Allocate CCCI header on stack.
 */
static inline
union ccci_hdr ccci_hdr_new(void)
{
	union ccci_hdr hdr;
	memset(hdr.raw, 0, 6);
	return hdr;
}

//
// Get/set version
//

static inline
uint8_t ccci_hdr_getversion(const union ccci_hdr *hdr)
{
	return hdr->raw[0];
}

static inline
void ccci_hdr_setversion(union ccci_hdr *hdr, uint8_t version)
{
	hdr->raw[0] = version;
}

//
// Get/set probe id
//

static inline
uint32_t ccci_hdr_getprobeid(const union ccci_hdr *hdr)
{
	return le_to_native(*((uint32_t *) (hdr->raw + 2)));
}

static inline
void ccci_hdr_setprobeid(union ccci_hdr *hdr, uint32_t probeid)
{
	uint32_t *raw32 = (uint32_t *) (hdr->raw + 2);
	*raw32 = native_to_le(probeid);
}

#endif

