/**
 * ccci_msg.h
 * Copyright (C) 2012, 2013 Jan Viktorin
 */

#ifndef CCCI_MSG_H
#define CCCI_MSG_H

#include "compat_endian.h"
#include "ccci_rule.h"

#include <inttypes.h>
#include <string.h>
#include <assert.h>

#define CCCI_MSG_SIZE   (4 + (CCCI_RULE_SIZE) + 4 + 4)
#define CCCI_MSG8_SIZE  (CCCI_MSG_SIZE)
#define CCCI_MSG16_SIZE (CCCI_MSG_SIZE / 2)
#define CCCI_MSG32_SIZE (CCCI_MSG_SIZE / 4)

/**
 * Representation of CCCI message.
 * It is suitable for network operations. The data
 * semantics is defined by setters and getters.
 */
union ccci_msg {
	uint8_t  raw  [CCCI_MSG_SIZE];
	uint16_t raw16[CCCI_MSG16_SIZE];
	uint32_t raw32[CCCI_MSG32_SIZE];
};

//
// Well-known offsets
//

#define CCCI_MSG8_ACTION   0
#define CCCI_MSG8_RULE     4
#define CCCI_MSG16_STATUS  1
#define CCCI_MSG32_RID    11
#define CCCI_MSG32_SID    12

/**
 * Allocation of a ccci_msg on stack.
 */
static inline
union ccci_msg ccci_msg_new(void)
{
	assert(sizeof(union ccci_msg) == CCCI_MSG_SIZE);

	union ccci_msg msg;
	memset(msg.raw, 0, CCCI_MSG_SIZE);
	return msg;
}

//
// Get/set action
//

static inline
uint8_t ccci_msg_getaction(const union ccci_msg *msg)
{
	return msg->raw[CCCI_MSG8_ACTION];
}

static inline
void ccci_msg_setaction(union ccci_msg *msg, uint8_t action)
{
	msg->raw[CCCI_MSG8_ACTION] = action;
}

//
// Get/set status
//

static inline
uint16_t ccci_msg_getstatus(const union ccci_msg *msg)
{
	return le_to_native16(msg->raw16[CCCI_MSG16_STATUS]);
}

static inline
void ccci_msg_setstatus(union ccci_msg *msg, uint16_t status)
{
	msg->raw16[CCCI_MSG16_STATUS] = native_to_le(status);
}

//
// Get rule
//

static inline
union ccci_rule *ccci_msg_getrule(const union ccci_msg *msg)
{
	return (union ccci_rule *) (msg->raw + CCCI_MSG8_RULE);
}

//
// Get/set rid
//

static inline
uint32_t ccci_msg_getrid(const union ccci_msg *msg)
{
	return le_to_native(msg->raw32[CCCI_MSG32_RID]);
}

static inline
void ccci_msg_setrid(union ccci_msg *msg, uint32_t rid)
{
	msg->raw32[CCCI_MSG32_RID] = native_to_le(rid);
}

//
// Get/set sid
//

static inline
uint32_t ccci_msg_getsid(const union ccci_msg *msg)
{
	return le_to_native(msg->raw32[CCCI_MSG32_SID]);
}

static inline
void ccci_msg_setsid(union ccci_msg *msg, uint32_t sid)
{
	msg->raw32[CCCI_MSG32_SID] = native_to_le(sid);
}

#endif
