/**
 * ccci_rule.h
 * Copyright (C) 2012, 2013 Jan Viktorin
 */

#ifndef CCCI_RULE_H
#define CCCI_RULE_H

#include "compat_endian.h"
#include "ccci_types.h"
#include <inttypes.h>

#define CCCI_RULE_SIZE   (4 + 16 + 16 + 2 + 2)
#define CCCI_RULE8_SIZE  (CCCI_RULE_SIZE)
#define CCCI_RULE16_SIZE (CCCI_RULE_SIZE / 2)
#define CCCI_RULE32_SIZE (CCCI_RULE_SIZE / 4)

/**
 * Representation of CCCI rule.
 * It is suitable for network operations. The data
 * semantics is defined by setters and getters.
 */
union ccci_rule {
	uint8_t  raw  [CCCI_RULE_SIZE];
	uint16_t raw16[CCCI_RULE16_SIZE];
	uint32_t raw32[CCCI_RULE32_SIZE];
};

//
// Well-known offsets
//

#define CCCI_RULE8_TYPE    0
#define CCCI_RULE8_IPVER   1
#define CCCI_RULE8_IPMASK  2
#define CCCI_RULE8_PROTO4  3
#define CCCI_RULE32_IP0    1
#define CCCI_RULE32_IP1    5
#define CCCI_RULE16_PORT0 18
#define CCCI_RULE16_PORT1 19

//
// Get/set type
//

static inline
uint8_t ccci_rule_gettype(const union ccci_rule *rule)
{
	return rule->raw[CCCI_RULE8_TYPE];
}

static inline
void ccci_rule_settype(union ccci_rule *rule, uint8_t type)
{
	rule->raw[CCCI_RULE8_TYPE] = type;
}

//
// Get/set IP version
//

static inline
uint8_t ccci_rule_getipver(const union ccci_rule *rule)
{
	return rule->raw[CCCI_RULE8_IPVER];
}

static inline
void ccci_rule_setipver(union ccci_rule *rule, uint8_t ipver)
{
	rule->raw[CCCI_RULE8_IPVER] = ipver;
}

//
// Get/set IP mask
//

static inline
uint8_t ccci_rule_getipmask(const union ccci_rule *rule)
{
	return rule->raw[CCCI_RULE8_IPMASK];
}

static inline
void ccci_rule_setipmask(union ccci_rule *rule, uint8_t ipmask)
{
	rule->raw[CCCI_RULE8_IPMASK] = ipmask;
}

//
// Get/set L4 protocol
//

static inline
uint8_t ccci_rule_getproto4(const union ccci_rule *rule)
{
	return rule->raw[CCCI_RULE8_PROTO4];
}

static inline
void ccci_rule_setproto4(union ccci_rule *rule, uint8_t proto4)
{
	rule->raw[CCCI_RULE8_PROTO4] = proto4;
}

//
// Get IP 0
//

static inline
union ccci_ipaddr *ccci_rule_getip0(const union ccci_rule *rule)
{
	return (union ccci_ipaddr *) (rule->raw32 + CCCI_RULE32_IP0);
}

//
// Get IP 1
//

static inline
union ccci_ipaddr *ccci_rule_getip1(const union ccci_rule *rule)
{
	return (union ccci_ipaddr *) (rule->raw32 + CCCI_RULE32_IP1);
}

//
// Get/set port 0
//

static inline
uint16_t ccci_rule_getport0(const union ccci_rule *rule)
{
	return le_to_native16(rule->raw16[CCCI_RULE16_PORT0]);
}

static inline
void ccci_rule_setport0(union ccci_rule *rule, uint8_t port0)
{
	rule->raw16[CCCI_RULE16_PORT0] = native_to_le16(port0);
}

//
// Get/set port 1
//

static inline
uint16_t ccci_rule_getport1(const union ccci_rule *rule)
{
	return le_to_native16(rule->raw16[CCCI_RULE16_PORT1]);
}

static inline
void ccci_rule_setport1(union ccci_rule *rule, uint8_t port1)
{
	rule->raw16[CCCI_RULE16_PORT1] = native_to_le16(port1);
}

#endif
