/**
 * ccci_types.h
 * Copyright (C) 2012, 2013 Jan Viktorin
 */

#ifndef CCCI_TYPES_H
#define CCCI_TYPES_H

#include "compat_endian.h"
#include <inttypes.h>

/**
 * CCCI header representation.
 */
union ccci_hdr;

/**
 * CCCI rule representation.
 */
union ccci_rule;

/**
 * CCCI message representation.
 */
union ccci_msg;

/**
 * CCCI IP address representation.
 * Provides both IPv4 and IPv6.
 */
union ccci_ipaddr {
	uint8_t  raw[16];
	uint16_t raw16[8];
	uint32_t raw32[4];
	uint64_t raw64[2];
};

//
// get IPv4
//

static inline
uint32_t ccci_ipaddr_getipv4(const union ccci_ipaddr *addr)
{
	return le_to_native(addr->raw32[0]);
}

//
// set IPv4
//

static inline
void ccci_ipaddr_setipv4(union ccci_ipaddr *addr, uint32_t v)
{
	addr->raw32[0] = native_to_le(v);
}

//
// get IPv6
//

static inline
void ccci_ipaddr_getipv6(const union ccci_ipaddr *addr, uint8_t v[16])
{
	for(int i = 0; i < 16; ++i)
		v[15 - i] = addr->raw[i];
}

static inline
void ccci_ipaddr_get32_ipv6(const union ccci_ipaddr *addr, uint32_t v[4])
{
	uint16_t *v16 = (uint16_t *) v;

	for(int i = 0; i < 8; ++i)
		v16[i] = le_to_native16(addr->raw16[i]);
}

//
// set IPv6
//

static inline
void ccci_ipaddr_setipv6(union ccci_ipaddr *addr, uint8_t v[16])
{
	for(int i = 0; i < 16; ++i)
		addr->raw[15 - i] = v[i];
}

static inline
void ccci_ipaddr_set32_ipv6(union ccci_ipaddr *addr, uint32_t v[4])
{
	uint16_t *v16 = (uint16_t *) v;

	for(int i = 0; i < 8; ++i)
		addr->raw16[i] = native_to_le16(v16[i]);
}

#endif
