/**
 * cccid_conf.h
 * Copyright (C) 2012, 2013 Jan Viktorin
 */

#ifndef CCCID_CONF_H
#define CCCID_CONF_H

#include <stddef.h>

struct cccid_conf {
	char **keys;
	char **values;
	int cap;
};

/**
 * Create new instance of cccid_conf on stack.
 */
static inline
struct cccid_conf cccid_conf_new(void)
{
	struct cccid_conf conf = {
		.keys = NULL,
		.values = NULL,
		.cap = 0
	};

	return conf;
}

/**
 * Initializes the configuration by argv.
 * Returns zero when successful.
 */
int cccid_conf_argv(struct cccid_conf *conf, int argc, char **argv);

/**
 * Release resources of the cccid_conf instance.
 */
void cccid_conf_fini(struct cccid_conf *conf);

/**
 * Returns value associated with the given key.
 * If no such value is available it returns the value
 * of defval.
 */
const char *cccid_conf_gets(struct cccid_conf *conf, const char *key,
		const char *defval);

/**
 * Returns value associated with the given key as integer.
 */
int cccid_conf_geti(struct cccid_conf *conf, const char *key,
		int defval);

#endif

