/**
 * cccid_conn.h
 * Copyright (C) 2012, 2013 Jan Viktorin
 */

#ifndef CCCID_CONN_H
#define CCCID_CONN_H

#include <stddef.h>
#include <inttypes.h>

/**
 * Connection representation.
 */
struct cccid_conn {
	int fd;
	/**
	 * Timeout for read/write operations.
	 */
	int timeout;
	/**
	 * Number of retries when connecting.
	 * Negative value means indefinitely.
	 */
	int retries;
	/**
	 * Delay in seconds between retries.
	 */
	int retry_delay;
};

static inline
struct cccid_conn cccid_conn_new(void)
{
	struct cccid_conn conn = {
		.fd = -1,
		.timeout = 0,
		.retries = 0,
		.retry_delay = 0
	};

	return conn;
}

/**
 * Connect to the remote host by TCP.
 * Returns zero when successful.
 */
int cccid_tcp_open(struct cccid_conn *conn, const char *host,
		const char *service);

/**
 * Close the connection.
 */
void cccid_conn_close(struct cccid_conn *conn);

/**
 * Sends the buffer over the connection.
 * Returns when all the buffer is sent.
 * Returns zero when successful.
 * Returns a negative value on failure.
 * Returns a positive value when interrupted
 * or when timeout exceeds.
 */
int cccid_conn_send(struct cccid_conn *conn,
		const uint8_t *b, size_t blen);

/**
 * Receives data over the connection.
 * Returns when all the buffer is filled.
 * Returns zero when successful.
 * Returns a negative value on failure.
 * Returns a positive value when interrupted
 * or when timeout exceeds.
 */
int cccid_conn_recv(struct cccid_conn *conn,
		uint8_t *b, size_t blen);

#endif

