/**
 * cccid_mod.c
 * Copyright (C) 2012, 2013 Jan Viktorin
 */

#include "cccid_mod.h"
#include "cccid_mod_gen.h"
#include <stddef.h>
#include <assert.h>

static
struct cccid_mod *head = NULL;

int cccid_mod_install(struct cccid_mod *mod)
{
	if(mod->match == NULL)
		return 1;

	if(mod->service == NULL)
		return 2;

	syslog(LOG_INFO, "Install mod_%s 0x%" PRIx16,
			mod->name, mod->version);

	mod->__next = head;
	head = mod;
	return 0;
}

void cccid_mod_install_all(struct cccid_conf *conf)
{
	__cccid_mod_install_all(conf);
}

static
struct cccid_mod *cccid_mod_uninstall(struct cccid_mod *mod)
{
	if(mod->fini)
		mod->fini();

	syslog(LOG_DEBUG, "Uninstall mod_%s 0x%" PRIx16,
			mod->name, mod->version);

	return mod->__next;
}

void cccid_mod_uninstall_all(void)
{
	while(head != NULL)
		head = cccid_mod_uninstall(head);
}

struct cccid_mod *cccid_mod_find_matching(struct cccid_mod *prev,
		const union ccci_msg *msg)
{
	struct cccid_mod *curr = prev == NULL? head : prev->__next;

	while(curr != NULL) {
		assert(curr->match != NULL);

		if(!curr->match(msg))
			return curr;

		syslog(LOG_DEBUG, "Skip mod_%s 0x%" PRIx16,
				curr->name, curr->version);
		curr = curr->__next;
	}

	return NULL;
}
