/**
 * cccid_mod.h
 * Copyright (C) 2012, 2013 Jan Viktorin
 */

#ifndef CCCID_MOD_H
#define CCCID_MOD_H

#include "ccci_types.h"
#include "cccid_conf.h"

#include <syslog.h>
#include <inttypes.h>

/**
 * Mark initialization function.
 */
#define cccid_mod_init(name, f) \
	void cccid__mod_##name##_init(struct cccid_conf *conf) {f(conf);}

/**
 * Finalize the module. Release all resources.
 */
typedef void (*cccid_mod_fini_f)(void);

/**
 * Determine whether the given message can be processed
 * by this module.
 * Returns 0 when the module will process the message.
 */
typedef int (*cccid_mod_match_f)(const union ccci_msg *);

/**
 * Handles the request and generates a response.
 * Returns 0 on success. No more processing is expected.
 * Returns positive when the request (response)
 *   processing can continue by another module.
 * Returns negative on failure (like no memory).
 */
typedef int (*cccid_mod_service_f)(const union ccci_msg *,
		union ccci_msg *);

/**
 * Module abstraction.
 */
struct cccid_mod {
	char *name;
	uint16_t version;
	cccid_mod_match_f   match;
	cccid_mod_service_f service;
	cccid_mod_fini_f    fini;
	// private member:
	struct cccid_mod *__next;
};

/**
 * Install the module into the running CCCI daemon.
 * Returns zero on success.
 */
int cccid_mod_install(struct cccid_mod *mod);

#endif
