/**
 * compat_endian.h
 * Copyright (C) 2013 Jan Viktorin
 */

#ifndef ENDIANITY_H
#define ENDIANITY_H

#ifdef __MICROBLAZEEL__

#include <inttypes.h>
#include <arpa/inet.h>

static inline uint32_t native_to_le(const uint32_t v) {return v;}
static inline uint32_t native_to_be(const uint32_t v) {return htonl(v);}

static inline uint32_t le_to_native(const uint32_t v) {return v;}
static inline uint32_t be_to_native(const uint32_t v) {return ntohl(v);}

static inline
uint32_t le_to_be(const uint32_t v)
{
	return native_to_be(le_to_native(v));
}

static inline
uint32_t be_to_le(const uint32_t v)
{
	return native_to_le(be_to_native(v));
}

static inline uint16_t native_to_le16(const uint16_t v) {return v;}
static inline uint16_t native_to_be16(const uint16_t v) {return htons(v);}

static inline uint16_t le_to_native16(const uint16_t v) {return v;}
static inline uint16_t be_to_native16(const uint16_t v) {return ntohs(v);}

static inline
uint16_t le_to_be16(const uint16_t v)
{
	return native_to_be16(le_to_native16(v));
}

static inline
uint16_t be_to_le16(const uint16_t v)
{
	return native_to_le16(be_to_native16(v));
}

#else

#define _BSD_SOURCE
#include <endian.h>
#include <inttypes.h>

static inline
uint32_t native_to_le(const uint32_t v)
{
	return htole32(v);
}

static inline
uint32_t native_to_be(const uint32_t v)
{
	return htobe32(v);
}

static inline
uint32_t le_to_native(const uint32_t v)
{
	return le32toh(v);
}

static inline
uint32_t be_to_native(const uint32_t v)
{
	return be32toh(v);
}

static inline
uint32_t le_to_be(const uint32_t v)
{
	return native_to_be(le_to_native(v));
}

static inline
uint32_t be_to_le(const uint32_t v)
{
	return native_to_le(be_to_native(v));
}

/////////////////////////////////////////////

static inline
uint16_t native_to_le16(const uint16_t v)
{
	return htole16(v);
}

static inline
uint16_t native_to_be16(const uint16_t v)
{
	return htobe16(v);
}

static inline
uint16_t le_to_native16(const uint16_t v)
{
	return le16toh(v);
}

static inline
uint16_t be_to_native16(const uint16_t v)
{
	return be16toh(v);
}

static inline
uint16_t le_to_be16(const uint16_t v)
{
	return native_to_be16(le_to_native16(v));
}

static inline
uint16_t be_to_le16(const uint16_t v)
{
	return native_to_le16(be_to_native16(v));
}

#endif

#endif

