/**
 * log_ccci.c
 * Copyright (C) 2013 Jan Viktorin
 */

#include "log_ccci.h"
#include <syslog.h>

static
void ccci_msg_print(int type, const union ccci_msg *msg)
{
	for(int i = 0; i < CCCI_MSG_SIZE; i += 4) {
		syslog(LOG_DEBUG, "%2d:"
				" %02" PRIx8 " %02" PRIx8
				" %02" PRIx8 " %02" PRIx8, i,
				msg->raw[i + 0], msg->raw[i + 1],
				msg->raw[i + 2], msg->raw[i + 3]);
	}

	if(type == 0)
		syslog(LOG_NOTICE, "Request: 0x%" PRIx8, ccci_msg_getaction(msg));
	else if(type == 1) {
		syslog(LOG_NOTICE, "Response: 0x%" PRIx8, ccci_msg_getaction(msg));
		syslog(LOG_NOTICE, "Result: %" PRIx16, ccci_msg_getstatus(msg));
	}
	syslog(LOG_NOTICE, "SID: 0x%" PRIx32, ccci_msg_getsid(msg));
	syslog(LOG_NOTICE, "RID: 0x%" PRIx32, ccci_msg_getrid(msg));

	union ccci_rule *rule = ccci_msg_getrule(msg);
	syslog(LOG_INFO, "Rule: 0x%" PRIx8, ccci_rule_gettype(rule));
	syslog(LOG_INFO, "IP Version: 0x%" PRIx8, ccci_rule_getipver(rule));
	syslog(LOG_INFO, "IP Mask: 0x%" PRIx8, ccci_rule_getipmask(rule));
	syslog(LOG_INFO, "L4 Protocol: 0x%" PRIx8, ccci_rule_getproto4(rule));

	union ccci_ipaddr *srcip = ccci_rule_getip0(rule);
	union ccci_ipaddr *dstip = ccci_rule_getip1(rule);

	if(ccci_rule_getipver(rule) == 4) {
		syslog(LOG_INFO, "IP0: 0x%" PRIx32, ccci_ipaddr_getipv4(srcip));
		syslog(LOG_INFO, "IP1: 0x%" PRIx32, ccci_ipaddr_getipv4(dstip));
	}
	else {
		uint8_t v[16];
		ccci_ipaddr_getipv6(srcip, v);
		syslog(LOG_INFO, "IP0: %02" PRIx8
				":%02" PRIx8 ":%02" PRIx8 ":%02" PRIx8
				":%02" PRIx8 ":%02" PRIx8 ":%02" PRIx8
				":%02" PRIx8 ":%02" PRIx8 ":%02" PRIx8
				":%02" PRIx8 ":%02" PRIx8 ":%02" PRIx8
				":%02" PRIx8 ":%02" PRIx8 ":%02" PRIx8,
				v[0], v[1], v[2], v[3], v[4], v[5],
				v[6], v[7], v[8], v[9], v[10], v[11],
				v[12], v[13], v[14], v[15]);

		ccci_ipaddr_getipv6(dstip, v);
		syslog(LOG_INFO, "IP1: %02" PRIx8
				":%02" PRIx8 ":%02" PRIx8 ":%02" PRIx8
				":%02" PRIx8 ":%02" PRIx8 ":%02" PRIx8
				":%02" PRIx8 ":%02" PRIx8 ":%02" PRIx8
				":%02" PRIx8 ":%02" PRIx8 ":%02" PRIx8
				":%02" PRIx8 ":%02" PRIx8 ":%02" PRIx8,
				v[0], v[1], v[2], v[3], v[4], v[5],
				v[6], v[7], v[8], v[9], v[10], v[11],
				v[12], v[13], v[14], v[15]);
	}

	syslog(LOG_INFO, "Port0: %" PRIx16, ccci_rule_getport0(rule));
	syslog(LOG_INFO, "Port1: %" PRIx16, ccci_rule_getport1(rule));
}

void log_ccci_request(const union ccci_msg *req)
{
	ccci_msg_print(0, req);
}

void log_ccci_response(const union ccci_msg *res)
{
	ccci_msg_print(1, res);
}
