/**
 * rid_mgmt.h
 * Copyright (C) 2012, 2013 Jan Viktorin
 */

#ifndef RID_MGMT_H
#define RID_MGMT_H

#include <stddef.h>
#include <inttypes.h>

typedef uint16_t rid_t;

struct rid_mgmt {
	void *bitmap;
	size_t max;
	rid_t *queue;
	size_t queue_begin;
	size_t queue_end;
	size_t queue_depth;
};

/**
 * Initialize the RID management for at most max count of possible
 * RID values allocated at once.
 * Returns zero on success.
 */
int rid_mgmt_init(struct rid_mgmt *mgmt, size_t max);

/**
 * Finalize the RID management.
 */
void rid_mgmt_fini(struct rid_mgmt *mgmt);

/**
 * Provide a unique RID value.
 * Returns 0 on success.
 */
int rid_mgmt_alloc(struct rid_mgmt *mgmt, rid_t *rid);

/**
 * Release the given RID and make it available for allocation.
 */
void rid_mgmt_release(struct rid_mgmt *mgmt, rid_t rid);

/**
 * Release the given RID but not immediately.
 * The RID is marked to be released but can not be allocated
 * for some implementation specific time.
 * We do not want the application to allocate RIDs that were
 * recently released.
 */
void rid_mgmt_release_later(struct rid_mgmt *mgmt, rid_t rid);

#endif
