/*!
 * \file hwio-cv2.c
 * \brief Implementation of libhwio using libcombo and libcommlbr
 * \author Lukas Kekely <xkekel00@stud.fit.vutbr.cz>
 * \date 2012
 *
 * Copyright (C) 2012 Brno University of Technology
 *
 * LICENSE TERMS
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of the Company nor the names of its contributors
 *    may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * ALTERNATIVELY, provided that this notice is retained in full, this
 * product may be distributed under the terms of the GNU General Public
 * License (GPL), in which case the provisions of the GPL apply INSTEAD
 * OF those given above.
 *
 * This software is provided ``as is'', and any express or implied
 * warranties, including, but not limited to, the implied warranties of
 * merchantability and fitness for a particular purpose are disclaimed.
 * In no event shall the company or contributors be liable for any
 * direct, indirect, incidental, special, exemplary, or consequential
 * damages (including, but not limited to, procurement of substitute
 * goods or services; loss of use, data, or profits; or business
 * interruption) however caused and on any theory of liability, whether
 * in contract, strict liability, or tort (including negligence or
 * otherwise) arising in any way out of the use of this software, even
 * if advised of the possibility of such damage.
 *
 * $Id$
 *
 */

#include "hwio.h"
#include <ctype.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h> 
#include <commlbr.h>
#include <combo.h>

cs_device_t *dev=NULL;
cs_space_t *space=NULL;

/** deinitialize HW **/
void hwio_deinit() {
    cs_design_free(dev);
    cs_detach(&dev);
}

/** initialize HW **/
int hwio_init(const char *fn, const char *dn) {
    cs_component_t *component;
    char *file=CS_PATH_DEV(0);
    if (cs_attach_noex(&dev,file)!=0)  /* attach device and map address space */
        return 1;
    if (fn!=NULL) { /* map according selected design.xml file */
        if (cs_design_reload(dev,CS_DESIGN_XML,fn)!=0) {
            hwio_deinit();
            return 2;
        }
    } else { /* map according loaded design */
        if (cs_design_reload(dev, CS_DESIGN_NONE, NULL)!=0) {
            hwio_deinit();
            return 3;
        }
    }
    if(cs_component_find(dev,&component,NULL,dn,0)!=0) { /* find component */
        hwio_deinit();
        return 4;
    }
    if (cs_component_space(component,&space)!= 0) { /* get component space */
        hwio_deinit();
        return 5;
    }
    return 0;
}

/** write 32 bits (4 Bytes) into HW **/
void hwio_write32(uint32_t addr, uint32_t data){
    /* printf("ADDR: %08x\tDATA: %08x\n",addr,data); */
    cs_space_write_4(dev,space,addr,data);
}

/** read 32 bits (4 Bytes) from HW **/
uint32_t hwio_read32(uint32_t addr){
    /* printf("ADDR: %08x\t DATA: %08x\n",addr,cs_space_read_4(dev,flt_space,addr)); */
    return cs_space_read_4(dev,space,addr);
}
