/*!
 * \file hwio.h
 * \brief Library for common communication interface to HW using libcombo or libdtree 
 * \author Lukas Kekely <xkekel00@stud.fit.vutbr.cz>
 * \date 2012
 *
 * Copyright (C) 2012 Brno University of Technology
 *
 * LICENSE TERMS
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of the Company nor the names of its contributors
 *    may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * ALTERNATIVELY, provided that this notice is retained in full, this
 * product may be distributed under the terms of the GNU General Public
 * License (GPL), in which case the provisions of the GPL apply INSTEAD
 * OF those given above.
 *
 * This software is provided ``as is'', and any express or implied
 * warranties, including, but not limited to, the implied warranties of
 * merchantability and fitness for a particular purpose are disclaimed.
 * In no event shall the company or contributors be liable for any
 * direct, indirect, incidental, special, exemplary, or consequential
 * damages (including, but not limited to, procurement of substitute
 * goods or services; loss of use, data, or profits; or business
 * interruption) however caused and on any theory of liability, whether
 * in contract, strict liability, or tort (including negligence or
 * otherwise) arising in any way out of the use of this software, even
 * if advised of the possibility of such damage.
 *
 * $Id$
 *
 */

#ifndef HWIO_H_INCLUDED
#define HWIO_H_INCLUDED
#include <stdint.h> 

/**
* Initialization of communication with HW.
* @param fn Path to HW description data source. If NULL is passed a default
*           value is used automatically.
* @param dn Name of component inside HW design.
* @return ec Returns 0 on success, or error code otherwise.
*/
int hwio_init(const char *fn, const char *dn);

/**
* Deinitialization of communication with HW. Must be performed before end of program.
*/
void hwio_deinit();

/**
* Writes 32b (4B) inside given HW component address space.
* @param addr Address offset inside HW component address space.
* @param data Data to be written.
*/
void hwio_write32(uint32_t addr, uint32_t data);

/**
* Reads 32b (4B) from HW component address space.
* @param addr Address offset inside HW component address space.
* @return data Data readed from filter.
*/
uint32_t hwio_read32(uint32_t addr);

#endif
