/**
 * @file hwio_cv2_comp.c
 * @brief Component functions for libcombo implementation
 * @author Denis Matoušek <xmatou08@stud.fit.vutbr.cz>
 * @copyright Brno University of Technology
 * @date 2012
 *
 * @section license LICENSE TERMS
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of the Company nor the names of its contributors
 *    may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * ALTERNATIVELY, provided that this notice is retained in full, this
 * product may be distributed under the terms of the GNU General Public
 * License (GPL), in which case the provisions of the GPL apply INSTEAD
 * OF those given above.
 *
 * This software is provided ``as is'', and any express or implied
 * warranties, including, but not limited to, the implied warranties of
 * merchantability and fitness for a particular purpose are disclaimed.
 * In no event shall the company or contributors be liable for any
 * direct, indirect, incidental, special, exemplary, or consequential
 * damages (including, but not limited to, procurement of substitute
 * goods or services; loss of use, data, or profits; or business
 * interruption) however caused and on any theory of liability, whether
 * in contract, strict liability, or tort (including negligence or
 * otherwise) arising in any way out of the use of this software, even
 * if advised of the possibility of such damage.
 */

#include "hwio_cv2.h"
#include "hwio_private.h"

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <combo.h>

struct hwio_comp *hwio_comp_alloc(void)
{
	const size_t siz = sizeof(struct hwio_comp);
	struct hwio_comp *comp = malloc(siz);

	if (comp == NULL)
		return NULL;

	return memset(comp, 0, siz);
}

void hwio_plat_comp_free(struct hwio_comp *comp)
{
	if (comp == NULL)
		return;

	if (comp->vendor != NULL)
		free(comp->vendor);

	if (comp->type != NULL)
		free(comp->type);

	if (comp->name != NULL)
		free(comp->name);
}

int hwio_comp_write(struct hwio_comp *comp, uint32_t offset, uint32_t data)
{
	if (offset % 4 != 0)
		return HWIO_EINVAL;

	cs_space_t *cspace;
	cs_component_space((cs_component_t *) comp->plat_comp, &cspace);
	cs_space_write_4((cs_device_t *) comp->dev->plat_dev, cspace,
		offset, data);

	return HWIO_EOK;
}

int hwio_comp_read(struct hwio_comp *comp, uint32_t offset, uint32_t *data)
{
	if (offset % 4 != 0)
		return HWIO_EINVAL;

	cs_space_t *cspace;
	cs_component_space((cs_component_t *) comp->plat_comp, &cspace);
	*data = cs_space_read_4((cs_device_t *) comp->dev->plat_dev, cspace,
		offset);

	return HWIO_EOK;
}
