/**
 * @file hwio_cv2_dev.c
 * @brief Device functions for libcombo implementation
 * @author Denis Matoušek <xmatou08@stud.fit.vutbr.cz>
 * @copyright Brno University of Technology
 * @date 2012
 *
 * @section license LICENSE TERMS
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of the Company nor the names of its contributors
 *    may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * ALTERNATIVELY, provided that this notice is retained in full, this
 * product may be distributed under the terms of the GNU General Public
 * License (GPL), in which case the provisions of the GPL apply INSTEAD
 * OF those given above.
 *
 * This software is provided ``as is'', and any express or implied
 * warranties, including, but not limited to, the implied warranties of
 * merchantability and fitness for a particular purpose are disclaimed.
 * In no event shall the company or contributors be liable for any
 * direct, indirect, incidental, special, exemplary, or consequential
 * damages (including, but not limited to, procurement of substitute
 * goods or services; loss of use, data, or profits; or business
 * interruption) however caused and on any theory of liability, whether
 * in contract, strict liability, or tort (including negligence or
 * otherwise) arising in any way out of the use of this software, even
 * if advised of the possibility of such damage.
 */

#include "hwio_cv2.h"
#include "hwio_private.h"

#include <stddef.h>
#include <combo.h>

const char *hwio_default_dev_phys(void)
{
	return CS_PATH_DEV(0);
}

const char *hwio_default_dev_src(void)
{
	return "Design.xml";
}

int hwio_init(struct hwio_dev *dev, const char *phys, const char *src)
{
	/* if NULL, use the first card */
	if (phys == NULL || src == NULL)
		return HWIO_EINVAL;

	cs_device_t *cdev = NULL;
	if (cs_attach_noex(&cdev, phys) != 0)
		return HWIO_EATT;

	dev->plat_dev = cdev;

	if (cs_design_reload(cdev, CS_DESIGN_XML, src) != 0) {
		cs_detach(&cdev);
		return HWIO_EINFO;
	}

	return HWIO_EOK;
}

void hwio_deinit(struct hwio_dev *dev)
{
	cs_device_t *cdev = (cs_device_t *) dev->plat_dev;
	/* cs_degisn_free always returns 0, it ignores
	 * return values of functions that it calls
	 */
	(void) cs_design_free(cdev);
	cs_detach(&cdev);
}
