/**
 * @file hwio_debug_comp.c
 * @brief Component functions for debug implementation
 * @author Denis Matoušek <xmatou08@stud.fit.vutbr.cz>
 * @copyright Brno University of Technology
 * @date 2012
 *
 * @section license LICENSE TERMS
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of the Company nor the names of its contributors
 *    may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * ALTERNATIVELY, provided that this notice is retained in full, this
 * product may be distributed under the terms of the GNU General Public
 * License (GPL), in which case the provisions of the GPL apply INSTEAD
 * OF those given above.
 *
 * This software is provided ``as is'', and any express or implied
 * warranties, including, but not limited to, the implied warranties of
 * merchantability and fitness for a particular purpose are disclaimed.
 * In no event shall the company or contributors be liable for any
 * direct, indirect, incidental, special, exemplary, or consequential
 * damages (including, but not limited to, procurement of substitute
 * goods or services; loss of use, data, or profits; or business
 * interruption) however caused and on any theory of liability, whether
 * in contract, strict liability, or tort (including negligence or
 * otherwise) arising in any way out of the use of this software, even
 * if advised of the possibility of such damage.
 */

#include "hwio_debug.h"
#include "hwio_private.h"

#include <stdint.h> 
#include <stdlib.h> 
#include <string.h>

struct hwio_comp *hwio_comp_alloc(void)
{
	const size_t siz = sizeof(struct hwio_comp)
	                 + sizeof(struct hwio_debug_comp);
	struct hwio_comp *comp = malloc(siz);
	if (comp == NULL)
		return NULL;

	memset(comp, 0, siz);
	struct hwio_debug_comp *debug = (struct hwio_debug_comp *) (comp + 1);
	comp->plat_comp = debug;

	return comp;
}

void hwio_plat_comp_free(struct hwio_comp *comp)
{
	if (comp == NULL)
		return;

	if (comp->vendor != NULL)
		free(comp->vendor);

	if (comp->type != NULL)
		hwio_xml_free(comp->type);

	if (comp->name != NULL)
		hwio_xml_free(comp->name);
}

int hwio_comp_write(struct hwio_comp *comp, uint32_t offset, uint32_t data)
{
	struct hwio_plat_file *plat_file = comp->dev->plat_dev;
	uint32_t address = hwio_debug_base(comp) + offset;

	if (offset % 4 != 0)
		return HWIO_EINVAL;

	if (address >= plat_file->memory_size)
		return HWIO_EFAULT;

	uint32_t *memory = (uint32_t *) (plat_file->memory);
	memory[address >> 2] = data;

	return HWIO_EOK;
}

int hwio_comp_read(struct hwio_comp *comp, uint32_t offset, uint32_t *data)
{
	struct hwio_plat_file *plat_file = comp->dev->plat_dev;
	uint32_t address = hwio_debug_base(comp) + offset;

	if (offset % 4 != 0)
		return HWIO_EINVAL;

	if (address >= plat_file->memory_size)
		return HWIO_EFAULT;

	uint32_t *memory = (uint32_t *) (plat_file->memory);
	*data = memory[address >> 2];

	return HWIO_EOK;
}
