/**
 * @file hwio.h
 * @brief Platform independent interface to HW (Public API)
 * @author Denis Matoušek <xmatou08@stud.fit.vutbr.cz>
 * @copyright Brno University of Technology
 * @date 2012
 *
 * @section license LICENSE TERMS
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of the Company nor the names of its contributors
 *    may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * ALTERNATIVELY, provided that this notice is retained in full, this
 * product may be distributed under the terms of the GNU General Public
 * License (GPL), in which case the provisions of the GPL apply INSTEAD
 * OF those given above.
 *
 * This software is provided ``as is'', and any express or implied
 * warranties, including, but not limited to, the implied warranties of
 * merchantability and fitness for a particular purpose are disclaimed.
 * In no event shall the company or contributors be liable for any
 * direct, indirect, incidental, special, exemplary, or consequential
 * damages (including, but not limited to, procurement of substitute
 * goods or services; loss of use, data, or profits; or business
 * interruption) however caused and on any theory of liability, whether
 * in contract, strict liability, or tort (including negligence or
 * otherwise) arising in any way out of the use of this software, even
 * if advised of the possibility of such damage.
 *
 * @mainpage
 *
 * The library libhwio is a part of the sec6net project
 * https://merlin.fit.vutbr.cz/wiki-sec6net/.
 *
 * It implements an abstract layer for accessing hardware components.
 *
 * @section mainpage-public-api Public API
 *
 * The public API is accessible via hwio.h header file that includes
 * all necessary header files.  Each public API header file has a note
 * “(Public API)” in its brief description.  For detailed description
 * see the documentation of the appropriate public API header file.
 *
 * @section mainpage-hw-support HW Support
 *
 * Now, these architectures are supported:
 * - COMBOv2 for high-speed probe,
 * - ug4-150 for micro probe and
 * - debugger for testing
 *
 * @section mainpage-additional-doc Additional Documentation
 *
 * Additional documentation on usage of the library and some comments
 * for developers can be found in the private sec6net wiki
 * https://merlin.fit.vutbr.cz/wiki-sec6net/index.php/Knihovna_hwio.
 */

#ifndef __HWIO_H__
#define __HWIO_H__

#define HWIO_VERSION "0.4"

#include "hwio_error.h"
#include "hwio_version.h"
#include "hwio_dev.h"
#include "hwio_comp_spec.h"
#include "hwio_comp.h"
#include "hwio_query.h"

#endif // __HWIO_H__
