/**
 * @file hwio_comp_spec.h
 * @brief Component specification definitions (Public API)
 * @author Denis Matoušek <xmatou08@stud.fit.vutbr.cz>
 * @author Jan Viktorin <xvikto03@stud.fit.vutbr.cz>
 * @copyright Brno University of Technology
 * @date 2012
 *
 * @section license LICENSE TERMS
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of the Company nor the names of its contributors
 *    may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * ALTERNATIVELY, provided that this notice is retained in full, this
 * product may be distributed under the terms of the GNU General Public
 * License (GPL), in which case the provisions of the GPL apply INSTEAD
 * OF those given above.
 *
 * This software is provided ``as is'', and any express or implied
 * warranties, including, but not limited to, the implied warranties of
 * merchantability and fitness for a particular purpose are disclaimed.
 * In no event shall the company or contributors be liable for any
 * direct, indirect, incidental, special, exemplary, or consequential
 * damages (including, but not limited to, procurement of substitute
 * goods or services; loss of use, data, or profits; or business
 * interruption) however caused and on any theory of liability, whether
 * in contract, strict liability, or tort (including negligence or
 * otherwise) arising in any way out of the use of this software, even
 * if advised of the possibility of such damage.
 */

#ifndef _HWIO_COMP_SPEC_H
#define _HWIO_COMP_SPEC_H

#include "hwio_version.h"

/**
 * Terminating item of a list of compatible components.
 */
#define HWIO_COMPAT_END {NULL, NULL, {0, 0, 0}}

/**
 * If the vendor of a component is not known
 * then this string is used for that.
 */
#define HWIO_VENDOR_UNKNOWN "_UNKNOWN_VENDOR_"

/**
 * If the type of a component is not known
 * then this string is used for that.
 */
#define HWIO_TYPE_UNKNOWN "_UNKNOWN_TYPE_"

/**
 * Component specification structure.
 */
struct hwio_comp_spec {
	char *vendor;
	char *type;
	struct hwio_version version;
};

#endif /* _HWIO_COMP_SPEC_H */
