/**
 * @file hwio_dev.h
 * @brief Representation of a device (Public API)
 * @author Denis Matoušek <xmatou08@stud.fit.vutbr.cz>
 * @author Jan Viktorin <xvikto03@stud.fit.vutbr.cz>
 * @copyright Brno University of Technology
 * @date 2012
 *
 * @section license LICENSE TERMS
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of the Company nor the names of its contributors
 *    may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * ALTERNATIVELY, provided that this notice is retained in full, this
 * product may be distributed under the terms of the GNU General Public
 * License (GPL), in which case the provisions of the GPL apply INSTEAD
 * OF those given above.
 *
 * This software is provided ``as is'', and any express or implied
 * warranties, including, but not limited to, the implied warranties of
 * merchantability and fitness for a particular purpose are disclaimed.
 * In no event shall the company or contributors be liable for any
 * direct, indirect, incidental, special, exemplary, or consequential
 * damages (including, but not limited to, procurement of substitute
 * goods or services; loss of use, data, or profits; or business
 * interruption) however caused and on any theory of liability, whether
 * in contract, strict liability, or tort (including negligence or
 * otherwise) arising in any way out of the use of this software, even
 * if advised of the possibility of such damage.
 */

#ifndef _HWIO_DEV_H
#define _HWIO_DEV_H

/**
 * Device structure.
 */
struct hwio_dev {
	void *plat_dev; // platform-dependent data (cs_device_t...)
};

/**
 * Initialization of the library. Platform dependent.
 * @param dev Platform-independent device structure to initialize.
 * @param phys Physical device file. The default value can be obtained
 *   by hwio_default_dev_phys().
 * @param src Design description file. The default value can be obtained
 *   by hwio_default_dev_src().
 * @return Returns HWIO_EOK on success,
 *   HWIO_ENOMEM on allocation error,
 *   HWIO_EATT on error while attaching a device,
 *   HWIO_EINFO on error while getting device info
*/
int hwio_init(struct hwio_dev *dev, const char *phys, const char *src);

/**
 * Deinitialization of the library. Platform dependent.
 */
void hwio_deinit(struct hwio_dev *dev);

/**
 * Returns the implementation specific default phys.
 * If an implementation can not provide such string,
 * then it can return NULL and MUST accept that NULL.
 */
const char *hwio_default_dev_phys(void);

/**
 * Returns the implementation specific default src.
 * If an implementation can not provide such string,
 * then it can return NULL and MUST accept that NULL.
 */
const char *hwio_default_dev_src(void);

#endif /* _HWIO_DEV_H */
