/**
 * @file hwio_query.h
 * @brief Query specification definitions (Public API)
 * @author Denis Matoušek <xmatou08@stud.fit.vutbr.cz>
 * @author Jan Viktorin <xvikto03@stud.fit.vutbr.cz>
 * @copyright Brno University of Technology
 * @date 2012
 *
 * @section license LICENSE TERMS
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of the Company nor the names of its contributors
 *    may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * ALTERNATIVELY, provided that this notice is retained in full, this
 * product may be distributed under the terms of the GNU General Public
 * License (GPL), in which case the provisions of the GPL apply INSTEAD
 * OF those given above.
 *
 * This software is provided ``as is'', and any express or implied
 * warranties, including, but not limited to, the implied warranties of
 * merchantability and fitness for a particular purpose are disclaimed.
 * In no event shall the company or contributors be liable for any
 * direct, indirect, incidental, special, exemplary, or consequential
 * damages (including, but not limited to, procurement of substitute
 * goods or services; loss of use, data, or profits; or business
 * interruption) however caused and on any theory of liability, whether
 * in contract, strict liability, or tort (including negligence or
 * otherwise) arising in any way out of the use of this software, even
 * if advised of the possibility of such damage.
 */

#ifndef _HWIO_QUERY_H
#define _HWIO_QUERY_H

#include "hwio_dev.h"
#include "hwio_comp_spec.h"

/**
 * Get the list of components specified by name and compatibility table.
 * @param dev Device to look for in.
 * @param spec Compatibility table - list of hwio_comp_spec instances finished by HWIO_COMPAT_END.
 * @return Returns HWIO_EOK on success,
 *   HWIO_ENOMEM on allocation error,
 *   HWIO_EINVAL when NULL was passed to dev of spec,
 *   or on invalid memory space.
 */
int hwio_byname(struct hwio_dev *dev, const char *name,
	const struct hwio_comp_spec *spec, struct hwio_comp **head);

/**
 * Get the list of components specified by compatibility table.
 * @param dev Device to look for in.
 * @param spec Compatibility table - list of hwio_comp_spec instances finished by HWIO_COMPAT_END.
 * @return Returns HWIO_EOK on success,
 *   HWIO_ENOMEM on allocation error,
 *   HWIO_EINVAL when NULL was passed to dev of spec,
 *   or on invalid memory space.
 */
int hwio_byspec(struct hwio_dev *dev, const struct hwio_comp_spec *spec,
	struct hwio_comp **head);

#endif /* _HWIO_QUERY_H */
