/**
 * @file hwio_test.h
 * @brief Common testing routines
 * @author Jan Viktorin <xvikto03@stud.fit.vutbr.cz>
 * @author Denis Matoušek <xmatou08@stud.fit.vutbr.cz>
 * @copyright Brno University of Technology
 * @date 2012
 *
 * @section license LICENSE TERMS
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of the Company nor the names of its contributors
 *    may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * ALTERNATIVELY, provided that this notice is retained in full, this
 * product may be distributed under the terms of the GNU General Public
 * License (GPL), in which case the provisions of the GPL apply INSTEAD
 * OF those given above.
 *
 * This software is provided ``as is'', and any express or implied
 * warranties, including, but not limited to, the implied warranties of
 * merchantability and fitness for a particular purpose are disclaimed.
 * In no event shall the company or contributors be liable for any
 * direct, indirect, incidental, special, exemplary, or consequential
 * damages (including, but not limited to, procurement of substitute
 * goods or services; loss of use, data, or profits; or business
 * interruption) however caused and on any theory of liability, whether
 * in contract, strict liability, or tort (including negligence or
 * otherwise) arising in any way out of the use of this software, even
 * if advised of the possibility of such damage.
 */

#ifndef HWIO_TEST_H
#define HWIO_TEST_H

#include <stdio.h>
#include "hwio_version.h"

#define test_start() fprintf(stderr, "  Started %s\n", __func__)
#define test_end()   fprintf(stderr, "  Finished %s\n", __func__)
#define subtest_start(name) fprintf(stderr, "  Started %s\n", (name))
#define subtest_end(name)   fprintf(stderr, "  Finished %s\n", (name))
#define test_fail(msg, r) fprintf(stderr, "! Failure at %s (%d): %s\n", __func__, (r), (msg))
#define test_assert(x, msg) if(!(x)) {fprintf(stderr, "! Assertion failed at %s (%s.%d): %s\n", __func__, __FILE__, __LINE__, (msg));}

#define test_version(ver, maj, min, sub) _test_version((ver), (maj), (min), (sub), __func__, __FILE__, __LINE__)
static inline
void _test_version(struct hwio_version ver, int maj, int min, int sub, const char *func, const char *file, int line)
{
	if(ver.major != maj)
		fprintf(stderr, "! Major doesn't match at %s (%s.%d): %d != %d\n", __func__, file, line, ver.major, maj);
	if(ver.minor != min)
		fprintf(stderr, "! Minor doesn't match at %s (%s.%d): %d != %d\n", __func__, file, line, ver.minor, min);
	if(ver.subminor != sub)
		fprintf(stderr, "! Subminor doesn't match at %s (%s.%d): %d != %d\n", __func__, file, line, ver.subminor, sub);
}

#endif

