/**
 * @file hwio_version.h
 * @brief Component version definitions (Public API)
 * @author Denis Matoušek <xmatou08@stud.fit.vutbr.cz>
 * @author Jan Viktorin <xvikto03@stud.fit.vutbr.cz>
 * @copyright Brno University of Technology
 * @date 2012
 *
 * @section license LICENSE TERMS
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of the Company nor the names of its contributors
 *    may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * ALTERNATIVELY, provided that this notice is retained in full, this
 * product may be distributed under the terms of the GNU General Public
 * License (GPL), in which case the provisions of the GPL apply INSTEAD
 * OF those given above.
 *
 * This software is provided ``as is'', and any express or implied
 * warranties, including, but not limited to, the implied warranties of
 * merchantability and fitness for a particular purpose are disclaimed.
 * In no event shall the company or contributors be liable for any
 * direct, indirect, incidental, special, exemplary, or consequential
 * damages (including, but not limited to, procurement of substitute
 * goods or services; loss of use, data, or profits; or business
 * interruption) however caused and on any theory of liability, whether
 * in contract, strict liability, or tort (including negligence or
 * otherwise) arising in any way out of the use of this software, even
 * if advised of the possibility of such damage.
 */

#ifndef _HWIO_VERSION_H
#define _HWIO_VERSION_H

/**
 * Version component not available.
 */
#define HWIO_VERSION_NA -1

#define HWIO_VER2(x, y) { .major = (x), .minor = (y), .subminor = 0 }
#define HWIO_VER3(x, y, z) { .major = (x), .minor = (y), .subminor = (z) }

/**
 * Version structure. Use `HWIO_VER*' macros to initialize.
 */
struct hwio_version {
	int major;
	int minor;
	int subminor;
};

/**
 * Helper (one-liner) to set the version.
 */
static inline
void hwio_version_set(struct hwio_version *v,
	int major, int minor, int subminor)
{
	v->major = major;
	v->minor = minor;
	v->subminor = subminor;
}

/**
 * Check the version against the version pattern.
 * @param version Version with specific values.
 * @param pattern Version with possible wildcards (HWIO_VERSION_NA).
 *   If some field is HWIO_VERSION_NA, it matches.
 * @return Returns 1 if the versions match, 0 otherwise.
 */
int hwio_version_match(struct hwio_version version,
	struct hwio_version pattern);

#endif /* _HWIO_VERSION */
