/**
 * @file xctl.c
 * @brief Example utility based on hwio.
 * @author Jan Viktorin <xvikto03@stud.fit.vutbr.cz>
 * @copyright Brno University of Technology
 * @date 2012
 *
 * @section license LICENSE TERMS
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of the Company nor the names of its contributors
 *    may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * ALTERNATIVELY, provided that this notice is retained in full, this
 * product may be distributed under the terms of the GNU General Public
 * License (GPL), in which case the provisions of the GPL apply INSTEAD
 * OF those given above.
 *
 * This software is provided ``as is'', and any express or implied
 * warranties, including, but not limited to, the implied warranties of
 * merchantability and fitness for a particular purpose are disclaimed.
 * In no event shall the company or contributors be liable for any
 * direct, indirect, incidental, special, exemplary, or consequential
 * damages (including, but not limited to, procurement of substitute
 * goods or services; loss of use, data, or profits; or business
 * interruption) however caused and on any theory of liability, whether
 * in contract, strict liability, or tort (including negligence or
 * otherwise) arising in any way out of the use of this software, even
 * if advised of the possibility of such damage.
 *
 * @mainpage
 *
 * The library libhwio is a part of the sec6net project
 * https://merlin.fit.vutbr.cz/wiki-sec6net/.
 */

#include "hwio_tool.h"
#include "hwio_comp_spec.h"

#include <stdio.h>
#include <string.h>

HWIO_TOOL_CUSTOM_NOT_USED

#define XCTL_OPTS "Z::T"

struct hwio_comp_spec xctl_compat[] = {
	{"debug", "IBUF", HWIO_VER2(2, 2)},
	{"debug", "OBUF", HWIO_VER2(1, 0)},
	{"debug", "PHYTERMDIO", HWIO_VER2(1, 0)},
	{"debug", "TSU_CV2", HWIO_VER2(2, 1)},
	{"debug", "Filter", HWIO_VER2(1, 0)},
	{"debug", "HFE-X", HWIO_VER2(1, 0)},
	HWIO_COMPAT_END
};

int xctl_custom_parse(int opt, const char *optarg,
		struct hwio_tool_params *params)
{
	switch(opt) {
	case 'T': // test the system
	case 'Z': // the magic option 'Z'
		if(hwio_tool_params_has_cmd(params))
			return hwio_tool_error_another_cmd(opt, params);

		params->cmd.code = opt;
		params->cmd.arg = optarg;

		return HWIO_TOOL_CUSTOM_OK;

	default:
		return HWIO_TOOL_CUSTOM_UNKNOWN;
	}
}

void hwio_tool_help(const char *toolname,
		const struct hwio_tool_params *params)
{
	printf("Usage: %s [options | command]\n", toolname);
	printf("Options:\n");
	printf("  -h\n");
	printf("  -v\n");
	printf("  -q\n");
	printf("  -x <path>\n");
	printf("  -d <path>\n");
	printf("  -A\n");
	printf("Options:\n");
	printf("  -L\n");
	printf("  -I\n");
	printf("  -E <state>\n");
	printf("  -Z [<string>]\n");
	printf("  -T\n");
}

int xctl_comp_enable(struct hwio_comp *c,
		hwio_tool_comp_err_t err, const struct hwio_tool_params *params)
{
	hwio_tool_verbose(params, 0, "The component '%s' has been %s", hwio_comp_name(c),
			strcmp(params->cmd.arg, "0")? "enabled" : "disabled");

	return HWIO_TOOL_OP_OK;
}

int xctl_comp_magic_z(struct hwio_comp *c,
		hwio_tool_comp_err_t err, const struct hwio_tool_params *params)
{
	hwio_tool_verbose(params, 0, "Performing the magic Z operation on component '%s': %s",
			hwio_comp_name(c), params->cmd.arg);
	return HWIO_TOOL_OP_OK;
}

int xctl_run_tests(const struct hwio_tool_params *params)
{
	return 0;
}

int main(int argc, char **argv)
{
	struct hwio_tool_params params = hwio_tool_params_new(
			HWIO_TOOL_CUSTOM_NULL, argc, argv);

	switch(hwio_tool_params_parse(&params, &xctl_custom_parse, XCTL_OPTS)) {
	case HWIO_TOOL_PARAMS_FAILED:
		return HWIO_TOOL_EARGS;

	case HWIO_TOOL_PARAMS_EXIT:
		return 0;

	default:
		break;
	}

	hwio_tool_verbose(&params, 0, "Arguments parsed successfully");
	hwio_tool_verbose(&params, 1, "Command: -%c", params.cmd.code);
	hwio_tool_verbose(&params, 2, "Argument: %s", params.cmd.arg);

	switch(params.cmd.code) {
	case 'I':
		return hwio_tool_exec_generic(&params, xctl_compat,
				&hwio_tool_comp_info, &hwio_tool_error_handler);

	case 'L':
		return hwio_tool_exec_generic(&params, xctl_compat,
				&hwio_tool_comp_name, &hwio_tool_error_handler);

	case 'E':
		return hwio_tool_exec_generic(&params, xctl_compat,
				&xctl_comp_enable, &hwio_tool_error_handler);

	case 'Z':
		return hwio_tool_exec_generic(&params, xctl_compat,
				&xctl_comp_magic_z, &hwio_tool_error_handler);

	case 'T':
		return xctl_run_tests(&params);

	case '\0':
		return hwio_tool_error(HWIO_TOOL_EARGS, "No command was specified");

	default:
		return hwio_tool_error(HWIO_TOOL_EARGS, "Command -%c is not implemented", params.cmd.code);
	}
}
