/*!
 */
#include <stdlib.h>
#include <stdio.h>
#include <libsze2.h>
#include <time.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
//#include <err.h>
#include <string.h>
#include <unistd.h>

#include <signal.h>

#include "compat.c"

#define LEN_CC_HEADER 16
#define UNKNOWN "unknown"


#define INI3_HEADER_LEN 6
#define INI3_VERSION_1 0x1
#define NONVALIDRID 0xFFFF

char server[1024]="127.0.0.1";
char port[10]="22224";
uint32_t probe_id=0x59EED;

void print_help(void)
{
        printf("sze2litcp\n");
        printf("=========\n");
        printf("Listens on sze interfaces, converts incoming sze frames into LI format\n");
        printf("and send them via TCP to distant MF server\n");
        printf("\n");
        printf("USAGE ./sze2litcp ARGUMENTS\n");
        printf("    -h             Print help\n");
        printf("    -s IPADDRESS   IP address of MF server\n");
        printf("    -p NUMBER      Destination port number\n");
        printf("    -i NUMBER      File with Probe ID\n");
    exit(EXIT_SUCCESS);
}

int arguments(int argc, char *argv[])
{
    // initial settings
    char opt;
    FILE *f;

    // argument parsing loop
    while ((opt = getopt(argc, argv, "hs:p:i:")) != -1) {
        switch (opt) {
        case 'h':  // help
            print_help();
            break;
        case 's':  // Server
            strcpy(server, optarg);
            break;
        case 'p':  // port
            strcpy(port, optarg);
            break;
        case 'i':  // port
            if ((f = fopen(optarg,"r")) != NULL) {
               fscanf(f, "%d", &probe_id);
               fclose(f);
            }
            else
               errx(1,"Unable to open file:%s",optarg);

            break;
        default:  // invalid arguments
            errx(1,"invalid arguments");
        }
    }
    return 1;
}

void signal_callback_handler(int signum) {

        if (signum == SIGINT) {
	exit(signum);
        }
}


void sockinfo(char *fmt, int fd)
{
	union {
	    struct sockaddr_in sin;
	    struct sockaddr_in6 sin6;
	    struct sockaddr sa;
	} socket;
	union {
	    char in[INET_ADDRSTRLEN];
	    char in6[INET6_ADDRSTRLEN];
	    char a[sizeof(UNKNOWN)];
	} addr[2];
	int port[2], i, length;

	for (i = 0; i < 2; i++) {
		length = sizeof(socket);
		if (((i == 0) ? getsockname : getpeername)(fd,
		    (struct sockaddr *)&socket, &length) == -1)
			err(1, (i == 0) ? "getsockname()" : "getpeername()");
		switch (socket.sa.sa_family) {
		case AF_INET:
			if (inet_ntop(AF_INET, &socket.sin.sin_addr,
			    addr[i].in, sizeof(addr[i].in)) == NULL)
				err(1, "inet_ntop()");
			port[i] = ntohs(socket.sin.sin_port);
			break;
		case AF_INET6:
			if (inet_ntop(AF_INET6, &socket.sin6.sin6_addr,
			    addr[i].in6, sizeof(addr[i].in6)) == NULL)
				err(1, "inet_ntop()");
			port[i] = ntohs(socket.sin6.sin6_port);
			break;
		default:
			strlcpy(addr[i].a, UNKNOWN, sizeof(addr[i].a));
			port[i] = 0;
			break;
		}
	}
	printf(fmt, addr[0].a, port[0], addr[1].a, port[1]);
}
void send_ini3_header(int servicefd)
{
        int r;
        
	unsigned char ini3_header[INI3_HEADER_LEN];
        ini3_header[0] = INI3_VERSION_1;
        ini3_header[1] = 0x0;
        strncpy(ini3_header+2, (unsigned char *) (&probe_id), 4); 

        r = write(servicefd, ini3_header, INI3_HEADER_LEN);
	if (r == -1) err(1, "write()");
	if (r != (INI3_HEADER_LEN ) ) errx(1, "write(): Buffer written just partially");


}

void service(unsigned char *packet, unsigned int packet_len, int servicefd)
{
	unsigned char  *data, *hw_data;
	unsigned int   data_len, hw_data_len;
	unsigned int   segsize;
	uint16_t blob_size;
	uint16_t sze_rid1  = 0x0;
	uint16_t sze_rid2  = 0x0;
	uint8_t  sze_iface = 0x0;
	uint8_t  sze_flags = 0x3;
	uint32_t sze_timestamp_sec;
	uint32_t sze_timestamp_ns;
        int            i, r;


        segsize = szedata_decode_packet(packet, &data, &hw_data, &data_len, &hw_data_len);

       
//        printf("packet hw data %04u B: ", hw_data_len);
//        for(i = 0; i < hw_data_len; i++)
//             printf("0x%02x ", *(hw_data + i));

        blob_size = ((uint16_t) (data_len));
        sze_iface = (uint8_t) (*(hw_data));
        sze_timestamp_ns =  (*((uint32_t *) (hw_data+4)));
        sze_timestamp_sec  =  (*((uint32_t *) (hw_data+8)));
        sze_rid1 = *((uint16_t *) (hw_data+12));
        sze_rid2 = *((uint16_t *) (hw_data+14));

        if (sze_rid1 == NONVALIDRID) {
           sze_rid1 = sze_rid2;
	   sze_flags = 0x2;
        }
        if (sze_rid2 == NONVALIDRID) {
           sze_rid2 = sze_rid1;
	   sze_flags = 0x1;
        }

        printf("blob_size: %04u B \n", blob_size);
        printf("sze_iface: %d\n", sze_iface);
        printf("sze_flags: %d\n", sze_flags);
        printf("sze_timestamp_sec: %ld\n", sze_timestamp_sec);
        printf("sze_timestamp_ns: %ld\n", sze_timestamp_ns);
        printf("sze_rid1: %d\n", sze_rid1);
        printf("sze_rid2: %d\n", sze_rid2);
        for(i = 0; i < data_len; i++)
             printf("0x%02x ", *(data + i));
        printf("\n");


	r = write(servicefd, (unsigned char *) &blob_size, 2);
	if (r == -1) err(1, "write()");
	if (r != 2) errx(1, "write(): Buffer written just partially blob_size");
	r = write(servicefd, (unsigned char *) &sze_iface, 1);
	if (r == -1) err(1, "write()");
	if (r != 1) errx(1, "write(): Buffer written just partially sze_iface");
	r = write(servicefd, (unsigned char *) &sze_flags, 1);
	if (r == -1) err(1, "write()");
	if (r != 1) errx(1, "write(): Buffer written just partially sze_iface");
	r = write(servicefd, (unsigned char *) &sze_timestamp_sec, 4);
	if (r == -1) err(1, "write()");
	if (r != 4) errx(1, "write(): Buffer written just partially sze_sec");
	r = write(servicefd, (unsigned char *) &sze_timestamp_ns, 4);
	if (r == -1) err(1, "write()");
	if (r != 4) errx(1, "write(): Buffer written just partially sze_ns");
	r = write(servicefd, (unsigned char *) &sze_rid1, 2);
	if (r == -1) err(1, "write()");
	if (r != 2) errx(1, "write(): Buffer written just partially sze_rid1");
	r = write(servicefd, (unsigned char *) &sze_rid2, 2);
	if (r == -1) err(1, "write()");
	if (r != 2) errx(1, "write(): Buffer written just partially sze_rid2");

	r = write(servicefd, data, data_len);
	if (r == -1) err(1, "write()");
	if (r != data_len) errx(1, "write(): Buffer written just partially data");

}

int main(int argc, char **argv) {

	struct szedata *sze = NULL;
	char *sze_dev = "/dev/szedataII0";
	unsigned int rx = 0xFF, tx = 0x00;
	int ret;

	int connectfd;
	int gai_error;
	struct addrinfo hints, *res, *res0;


        if (!arguments(argc, argv))
            return EXIT_FAILURE;


        signal(SIGINT, signal_callback_handler);

	memset(&hints, 0, sizeof(hints));
	hints.ai_family = AF_UNSPEC;
	hints.ai_socktype = SOCK_STREAM;
	if ((gai_error = getaddrinfo(server, port, &hints, &res0)) != 0)
		errx(1, "getaddrinfo(): %s", gai_strerror(gai_error));
	connectfd = -1;
       
	for (res = res0; res != NULL; res = res->ai_next) {
		printf("socket(...)\n");
		if ((connectfd = socket(res->ai_family, res->ai_socktype,
		    res->ai_protocol)) == -1) {
			warn("socket()");
			connectfd = -1;
			continue;
		}

		printf("connect(...)\n");
		if (connect(connectfd, res->ai_addr, res->ai_addrlen) == -1) {
			close(connectfd);
			connectfd = -1;
			warn("connect()");
		} else
			break;
	}
	if (connectfd < 0)
		errx(1, "getaddrinfo(): Interface not found");

	sockinfo("service(...): %s/%d -> %s/%d\n", connectfd);

	/* initialize szedata2 */
	/* get sze device handle */
	sze = szedata_open(sze_dev);
	if (sze == NULL)
		err("szedata_open failed");

	/* set 5s timeout for szedata_read_next */
	SZE2_RX_POLL_TIMEOUT = 5000;

	printf("subscribing: RX interface mask-0x%02hx; TX interface mask-0x%02hx\n", rx, tx);
	printf("poll timeouts: RX %ums; TX %ums\n", SZE2_RX_POLL_TIMEOUT, SZE2_TX_POLL_TIMEOUT);

	/* subscribe wanted interfaces and set poll byte conuts */
	ret = szedata_subscribe3(sze, &rx, &tx);
	if (ret)
		abort();
	printf("subscribed: RX 0x%02hx; TX 0x%02hx\n\n", rx, tx);

	ret = szedata_start(sze);
	if (ret)
		abort();

	unsigned char *packet;
	unsigned int packet_len;
	unsigned int packet_cnt = 0;


        send_ini3_header(connectfd);
	while(1) {
		if((packet = szedata_read_next(sze, &packet_len)) != NULL) {
                        packet_cnt++;
	                service(packet, packet_len, connectfd);

		} else {
                        ;
			//printf("nothing to read\n");
		}
	}

	close(connectfd);
	szedata_close(sze);
	return 0;
}	


