
#include <sys/types.h>
#include <sys/select.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <ctype.h>
#include <err.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "compat.c"

#define BUFFER	(2024)
#define QUEUE	(2)
#define SLEEP	(10)
#define LISTENFDS (16)

#define UNKNOWN "unknown"

#define  LEN_0        (0)
#define  LEN_1        (1)
#define  SID1_0       (2)
#define  SID1_1       (3)
#define  SID1_2       (4)
#define  SID1_3       (5)
#define  SID2_0       (6)
#define  SID2_1       (7)
#define  SID2_2       (8)
#define  SID2_3       (9)
#define  TIMESTAMP_0  (10)
#define  TIMESTAMP_1  (11)
#define  TIMESTAMP_2  (12)
#define  TIMESTAMP_3  (13)
#define  DATA         (14)
        




void sockinfo(char *fmt, int fd)
{
	union {
	    struct sockaddr_in sin;
	    struct sockaddr_in6 sin6;
	    struct sockaddr sa;
	} socket;
	union {
	    char in[INET_ADDRSTRLEN];
	    char in6[INET6_ADDRSTRLEN];
	    char a[sizeof(UNKNOWN)];
	} addr[2];
	int port[2], i, length;

	for (i = 0; i < 2; i++) {
		length = sizeof(socket);
		if (((i == 0) ? getsockname : getpeername)(fd,
		    (struct sockaddr *)&socket, &length) == -1)
			err(1, (i == 0) ? "getsockname()" : "getpeername()");
		switch (socket.sa.sa_family) {
		case AF_INET:
			if (inet_ntop(AF_INET, &socket.sin.sin_addr,
			    addr[i].in, sizeof(addr[i].in)) == NULL)
				err(1, "inet_ntop()");
			port[i] = ntohs(socket.sin.sin_port);
			break;
		case AF_INET6:
			if (inet_ntop(AF_INET6, &socket.sin6.sin6_addr,
			    addr[i].in6, sizeof(addr[i].in6)) == NULL)
				err(1, "inet_ntop()");
			port[i] = ntohs(socket.sin6.sin6_port);
			break;
		default:
			strlcpy(addr[i].a, UNKNOWN, sizeof(addr[i].a));
			port[i] = 0;
			break;
		}
	}
	printf(fmt, addr[0].a, port[0], addr[1].a, port[1]);
}

void service(int fd)
{
        int parsing_state = LEN_0;
	int n, r;
	char buf[BUFFER];
        unsigned short net_data_len;
        unsigned int net_sid1;
        unsigned int net_sid2;
        int net_timestamp;
        unsigned short data_len;
        unsigned int sid1;
        unsigned int sid2;
        int timestamp;
        unsigned int data_offset;


	while ((n = read(fd, buf, BUFFER)) > 0) {
		for (r = 0; r < n; r++) {
                        if (r % 8 == 0)
                           printf("\n");
                        printf("0x%02x ", (unsigned char) *(buf + r));
                }
                printf("\n");
	}
	if (n == -1)
		err(1, "read()");
}

int main(int argc, char *argv[])
{
	int listenfd[LISTENFDS], connectfd;
	struct hostent *hostent;
	int listenfds, gai_error, i, maxfd;
	struct addrinfo hints, *res, *res0;
	fd_set rset, allset;

	if (argc != 3)
		errx(1, "Usage: %s <ANY|address> <port>", getprogname());

	memset(&hints, 0, sizeof(hints));
	hints.ai_family = AF_UNSPEC;
	hints.ai_socktype = SOCK_STREAM;
	hints.ai_flags = AI_PASSIVE;

	if ((gai_error = getaddrinfo(NULL, argv[2], &hints, &res0)) != 0)
		errx(1, "getaddrinfo(): %s", gai_strerror(gai_error));
	listenfds = 0;
	for (res = res0; res != NULL && listenfds < LISTENFDS;
	    res = res->ai_next) {
		if ((listenfd[listenfds] = socket(res->ai_family,
		    res->ai_socktype, res->ai_protocol)) == -1) {
			warn("socket()");
			connectfd = -1;
			continue;
		}
		if (strcasecmp(argv[1], "any") != 0) {
			if ((hostent = gethostbyname2(argv[1], res->ai_family))
			    == NULL) {
				warnx("gethostbyname2(): %s",
				    hstrerror(h_errno));
				close(listenfd[listenfds]);
				continue;
			} else {
				if (res->ai_family == AF_INET)
					memcpy(&((struct sockaddr_in *)res
					    ->ai_addr)->sin_addr,
					    hostent->h_addr,
					    hostent->h_length);
				else if (res->ai_family == AF_INET6)
					memcpy(&((struct sockaddr_in6 *)res
					    ->ai_addr)->sin6_addr,
					    hostent->h_addr,
					    hostent->h_length);
				else {
					warnx("gethostbyname2():"
					    " Unsupported family");
					close(listenfd[listenfds]);
					continue;
				}
			}
		}
		if (bind(listenfd[listenfds],
		    res->ai_addr, res->ai_addrlen) == -1)
			err(1, "bind()");
		if (listen(listenfd[listenfds], QUEUE) == -1)
			err(1, "listen()");
		listenfds++;
	}
	if (listenfds == 0)
		errx(1, "getaddrinfo(): Interface not found");
	freeaddrinfo(res0);

	maxfd = 0;
	FD_ZERO(&allset);
	for (i = 0; i < listenfds; i++) {
		if (listenfd[i] > maxfd)
			maxfd = listenfd[i];
		FD_SET(listenfd[i], &allset);
	}
	while (1) {
#ifdef USE_SLEEP
		printf("sleep(%d)\n", SLEEP);
		sleep(SLEEP);
#endif
		rset = allset;
		if (select(maxfd + 1, &rset, NULL, NULL, NULL) == -1)
			err(1, "select()");
		for (i = 0; i < listenfds; i++)
			if (FD_ISSET(listenfd[i], &rset)) {
				if ((connectfd = accept(listenfd[i], NULL,
				    NULL)) == -1)
					err(1, "accept()");
				sockinfo("service(...): %s/%d -> %s/%d\n",
				    connectfd);
				service(connectfd);
				close(connectfd);
			}
	}

	for (i = 0; i < listenfds; i++) {
		close(listenfd[i]);
	}

	return 0;
}

